# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
import json
import six
from django_ace.widgets import AceWidget


class JSONWidget(AceWidget):
    def render(self, name, value, attrs=None):
        if not isinstance(value, six.string_types):
            value = json.dumps(value, sort_keys=True, indent=4, separators=(',', ': '))
        return super(JSONWidget, self).render(name, value, attrs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from django.conf import settings

from .sync.apis import APISyncEngine
from .sync.consumers import ConsumerSyncEngine


def get_api_sync_engine():
    return APISyncEngine()


def get_consumer_sync_engine():
    return ConsumerSyncEngine()


def get_kong_client():
    from kong.client import KongAdminClient
    from kong.simulator import KongAdminSimulator

    api_url = getattr(settings, 'KONG_ADMIN_URL')
    simulator_enabled = getattr(settings, 'KONG_ADMIN_SIMULATOR')

    if simulator_enabled is True:
        return KongAdminSimulator(api_url)

    return KongAdminClient(api_url)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from enum import Enum, unique


@unique
class Plugins(Enum):
    ssl = 0
    keyauth = 1
    basicauth = 2
    oauth2 = 3
    ratelimiting = 4
    tcplog = 5
    udplog = 6
    filelog = 7
    httplog = 8
    cors = 9
    request_transformer = 10
    response_transformer = 11
    requestsizelimiting = 12

    @staticmethod
    def choices():
        return [(key, key) for key in sorted(Plugins.__members__.keys())]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
import logging
import os
logging.disable(logging.CRITICAL)

import environ
BASE_DIR = environ.Path(os.path.dirname(__file__))
env = environ.Env(
    KONG_ADMIN_SIMULATOR=(bool, False),
)
SITE_ROOT = BASE_DIR()
DEBUG = True
TEMPLATE_DEBUG = DEBUG
SECRET_KEY = 'K-pF++*|+9alwjd4834327E;a'

ALLOWED_HOSTS = []
INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    # https://github.com/DarioGT/django-jsonfield2
    'jsonfield2',

    # https://github.com/bradleyayers/django-ace
    'django_ace',

    'kong_admin'
)
MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(str(BASE_DIR), 'templates/')
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]
LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True
STATIC_URL = '/static/'

ROOT_URLCONF = 'tests.settings.urls'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from contextlib import closing

from django.db.models.signals import pre_save, pre_delete
from django.dispatch.dispatcher import receiver

from .models import APIReference, ConsumerReference, BasicAuthReference, KeyAuthReference, OAuth2Reference, \
    PluginConfigurationReference
from .factory import get_kong_client, get_api_sync_engine, get_consumer_sync_engine


@receiver(pre_save, sender=APIReference)
def before_saving_api(sender, instance, **kwargs):
    instance.synchronized = False


@receiver(pre_delete, sender=APIReference)
def before_delete_api(sender, instance, **kwargs):
    with closing(get_kong_client()) as client:
        get_api_sync_engine().withdraw(client, instance)


@receiver(pre_save, sender=ConsumerReference)
def before_saving_consumer(sender, instance, **kwargs):
    instance.synchronized = False


@receiver(pre_delete, sender=ConsumerReference)
def before_delete_consumer(sender, instance, **kwargs):
    with closing(get_kong_client()) as client:
        get_consumer_sync_engine().withdraw(client, instance)


@receiver(pre_save, sender=BasicAuthReference)
def before_saving_basic_auth(sender, instance, **kwargs):
    """
    We synchronize BasicAuthReference objects together with the consumer
    """
    instance.synchronized = False
    ConsumerReference.objects.filter(id=instance.consumer.id, synchronized=True).update(synchronized=False)


@receiver(pre_save, sender=KeyAuthReference)
def before_saving_key_auth(sender, instance, **kwargs):
    """
    We synchronize KeyAuthReference objects together with the consumer
    """
    instance.synchronized = False
    ConsumerReference.objects.filter(id=instance.consumer.id, synchronized=True).update(synchronized=False)


@receiver(pre_save, sender=OAuth2Reference)
def before_saving_oauth(sender, instance, **kwargs):
    """
    We synchronize OAuth2Reference objects together with the consumer
    """
    instance.synchronized = False
    ConsumerReference.objects.filter(id=instance.consumer.id, synchronized=True).update(synchronized=False)


@receiver(pre_save, sender=PluginConfigurationReference)
def before_saving_plugin_configuration(sender, instance, **kwargs):
    instance.synchronized = False


@receiver(pre_delete, sender=PluginConfigurationReference)
def before_delete_plugin_configuration(sender, instance, **kwargs):
    with closing(get_kong_client()) as client:
        get_api_sync_engine().plugins().withdraw(client, instance)


__all__ = [before_saving_api, before_delete_api, before_saving_consumer, before_delete_consumer,
           before_saving_basic_auth, before_saving_key_auth, before_saving_oauth, before_saving_plugin_configuration,
           before_delete_plugin_configuration]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from contextlib import closing

from django.utils.translation import ugettext_lazy as _
from django.contrib import admin, messages
from django.contrib.admin.views.decorators import staff_member_required
from django.http.response import HttpResponseRedirect
from jsonfield2.fields import JSONField

from .models import APIReference, PluginConfigurationReference, ConsumerReference, \
    BasicAuthReference, KeyAuthReference, OAuth2Reference
from .factory import get_kong_client
from .logic import synchronize_apis, synchronize_api, synchronize_plugin_configurations, \
    synchronize_plugin_configuration, synchronize_consumers, synchronize_consumer
from .contrib import CustomModelAdmin
from .widgets import JSONWidget


@staff_member_required
def synchronize_api_references(request, queryset=None):
    try:
        with closing(get_kong_client()) as client:
            queryset = synchronize_apis(client, queryset=queryset)
    except Exception as e:
        messages.add_message(
            request, messages.ERROR, 'Could not synchronize API References: %s' % str(e))
    else:
        messages.add_message(
            request, messages.SUCCESS, 'Successfully synchronized %d API References (it can take a while before the '
                                       'changes are visible!)' % queryset.count())
    return HttpResponseRedirect(request.META["HTTP_REFERER"])


@staff_member_required
def synchronize_api_reference(request, pk, toggle_enable=False):
    obj = APIReference.objects.get(id=pk)

    try:
        with closing(get_kong_client()) as client:
            synchronize_api(client, obj, toggle=toggle_enable)
    except Exception as e:
        messages.add_message(
            request, messages.ERROR, 'Could not sync API Reference: %s (was it published?)' % str(e))
    else:
        messages.add_message(
            request, messages.SUCCESS, 'Successfully synced API Reference (it can take a while before the '
                                       'changes are visible!)')

    return HttpResponseRedirect(request.META["HTTP_REFERER"])


@staff_member_required
def synchronize_plugin_configuration_references(request, queryset=None):
    try:
        with closing(get_kong_client()) as client:
            queryset = synchronize_plugin_configurations(client, queryset=queryset)
    except Exception as e:
        messages.add_message(
            request, messages.ERROR, 'Could not synchronize Plugin Configuration References: %s' % str(e))
    else:
        messages.add_message(
            request, messages.SUCCESS, 'Successfully synchronized %d Plugin Configuration References (it can take a '
                                       'while before the changes are visible!)' % queryset.count())

    return HttpResponseRedirect(request.META["HTTP_REFERER"])


@staff_member_required
def synchronize_plugin_configuration_reference(request, pk, toggle_enable=False):

    obj = PluginConfigurationReference.objects.get(id=pk)

    try:
        with closing(get_kong_client()) as client:
            obj = synchronize_plugin_configuration(client, obj, toggle=toggle_enable)
    except Exception as e:
        messages.add_message(
            request, messages.ERROR, 'Could not publish Plugin Configuration Reference: %s' % str(e))
    else:
        messages.add_message(
            request, messages.SUCCESS, 'Successfully published Plugin Configuration Reference (it can take a while '
                                       'before the changes are visible!)')

    return HttpResponseRedirect(request.META["HTTP_REFERER"])


@staff_member_required
def synchronize_consumer_references(request, queryset=None):
    try:
        with closing(get_kong_client()) as client:
            queryset = synchronize_consumers(client, queryset=queryset)
    except Exception as e:
        messages.add_message(
            request, messages.ERROR, 'Could not synchronize Consumer References: %s' % str(e))
    else:
        messages.add_message(
            request, messages.SUCCESS, 'Successfully synchronized %d Consumer References (it can take a while before '
                                       'the changes are visible!)' % queryset.count())

    return HttpResponseRedirect(request.META["HTTP_REFERER"])


@staff_member_required
def synchronize_consumer_reference(request, pk, toggle_enable=False):
    obj = ConsumerReference.objects.get(id=pk)

    try:
        with closing(get_kong_client()) as client:
            obj = synchronize_consumer(client, obj, toggle=toggle_enable)
    except Exception as e:
        messages.add_message(
            request, messages.ERROR, 'Could not sync Consumer Reference: %s (was it published?)' % str(e))
    else:
        messages.add_message(
            request, messages.SUCCESS, 'Successfully synced Consumer Reference (it can take a while before the '
                                       'changes are visible!)')

    return HttpResponseRedirect(request.META["HTTP_REFERER"])


def get_toggle_enable_caption(obj):
    return 'Disable' if obj.enabled else 'Enable'


class PluginConfigurationReferenceInline(admin.StackedInline):
    model = PluginConfigurationReference
    extra = 0
    fields = ('name', 'value', 'enabled', 'consumer')
    formfield_overrides = {
        JSONField: {'widget': JSONWidget(mode='json', width='800px', height='180px', theme='twilight')},
    }


class APIReferenceAdmin(CustomModelAdmin):
    list_display = ('target_url', 'name', 'public_dns', 'path', 'enabled', 'synchronized', 'kong_id')
    list_display_buttons = [{
        'caption': 'Synchronize',
        'url': 'sync-api-ref/',
        'view': synchronize_api_reference
    }, {
        'caption': get_toggle_enable_caption,
        'url': 'toggle-enable/',
        'view': lambda request, pk: synchronize_api_reference(request, pk, toggle_enable=True)
    }]
    action_buttons = [{
        'caption': 'Synchronize all',
        'url': 'sync-api-refs/',
        'view': synchronize_api_references
    }]
    list_select_related = True
    fieldsets = (
        (None, {
            'fields': ('target_url', 'name', 'public_dns', 'path', 'enabled')
        }),
        (_('Advanced options'), {
            'fields': ('strip_path',)
        }),
        (_('Audit'), {
            'fields': ('created_at', 'updated_at')
        }),
    )
    inlines = [
        PluginConfigurationReferenceInline
    ]
    readonly_fields = ('created_at', 'updated_at')

admin.site.register(APIReference, APIReferenceAdmin)


# class PluginConfigurationReferenceAdmin(CustomModelAdmin):
#     list_display = ('name', 'api', 'enabled', 'synchronized', 'kong_id')
#     list_display_buttons = [{
#         'caption': 'Synchronize',
#         'url': 'sync-plugin-configuration-ref/',
#         'view': synchronize_plugin_configuration_reference
#     }, {
#         'caption': get_toggle_enable_caption,
#         'url': 'toggle-enable/',
#         'view': lambda request, pk: synchronize_plugin_configuration_reference(request, pk, toggle_enable=True)
#     }]
#     action_buttons = [{
#         'caption': 'Synchronize all',
#         'url': 'sync-plugin-configuration-refs/',
#         'view': synchronize_plugin_configuration_references
#     }]
#     list_select_related = True
#     fieldsets = (
#         (None, {
#             'fields': ('name', 'enabled', 'value')
#         }),
#         (_('Target'), {
#             'fields': ('api', 'consumer')
#         }),
#         (_('Audit'), {
#             'fields': ('created_at', 'updated_at')
#         }),
#     )
#     readonly_fields = ('created_at', 'updated_at')
#
# admin.site.register(PluginConfigurationReference, PluginConfigurationReferenceAdmin)


class BasicAuthInline(admin.StackedInline):
    model = BasicAuthReference
    extra = 0
    fields = ('username', 'password')


class KeyAuthInline(admin.StackedInline):
    model = KeyAuthReference
    extra = 0
    fields = ('key',)


class OAuthInline(admin.StackedInline):
    model = OAuth2Reference
    extra = 0
    fields = ('name', 'redirect_uri', 'client_id', 'client_secret')


class ConsumerReferenceAdmin(CustomModelAdmin):
    list_display = ('username_or_custom_id', 'enabled', 'synchronized', 'kong_id')
    list_display_buttons = [{
        'caption': 'Synchronize',
        'url': 'sync-consumer-ref/',
        'view': synchronize_consumer_reference
    }, {
        'caption': get_toggle_enable_caption,
        'url': 'toggle-enable/',
        'view': lambda request, pk: synchronize_consumer_reference(request, pk, toggle_enable=True)
    }]
    action_buttons = [{
        'caption': 'Synchronize all',
        'url': 'sync-consumer-refs/',
        'view': synchronize_consumer_references
    }]
    list_select_related = True
    fieldsets = (
        (None, {
            'fields': ('username', 'custom_id', 'enabled')
        }),
        (_('Audit'), {
            'fields': ('created_at', 'updated_at')
        }),
    )
    readonly_fields = ('created_at', 'updated_at')
    inlines = [
        BasicAuthInline,
        KeyAuthInline,
        OAuthInline
    ]

    def username_or_custom_id(self, obj):
        return obj.username or obj.custom_id

admin.site.register(ConsumerReference, ConsumerReferenceAdmin)


# class OAuth2ApplicationAdmin(CustomModelAdmin):
#     list_display = ('name','redirect_uri', 'client_id', 'client_secret')
#     fieldsets = (
#         (None, {
#             'fields': ('name', 'redirect_uri')
#         }),
#         (_('Consumers'), {
#             'fields': ('consumers',)
#         }),
#         (_('Credentials'), {
#             'fields': ('client_id', 'client_secret')
#         }),
#     )
#     raw_id_fields = ('consumers',)
#     readonly_fields = ('created_at', 'updated_at')
#
# admin.site.register(OAuth2Application, OAuth2ApplicationAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='APIReference',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('kong_id', models.UUIDField(null=True, editable=False, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('synchronized', models.BooleanField(default=False)),
                ('synchronized_at', models.DateTimeField(null=True, editable=False, blank=True, verbose_name='synchronized')),
                ('target_url', models.URLField()),
                ('name', models.CharField(null=True, unique=True, default=None, max_length=32, blank=True)),
                ('public_dns', models.CharField(null=True, unique=True, default=None, max_length=32, blank=True)),
                ('path', models.CharField(null=True, default=None, max_length=32, blank=True)),
                ('strip_path', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name_plural': 'API References',
                'verbose_name': 'API Reference',
            },
        ),
        migrations.CreateModel(
            name='BasicAuthReference',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('kong_id', models.UUIDField(null=True, editable=False, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('synchronized', models.BooleanField(default=False)),
                ('synchronized_at', models.DateTimeField(null=True, editable=False, blank=True, verbose_name='synchronized')),
                ('username', models.CharField(max_length=32, unique=True)),
                ('password', models.CharField(max_length=40)),
            ],
            options={
                'verbose_name_plural': 'Basic Auth References',
                'verbose_name': 'Basic Auth Reference',
            },
        ),
        migrations.CreateModel(
            name='ConsumerReference',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('kong_id', models.UUIDField(null=True, editable=False, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('synchronized', models.BooleanField(default=False)),
                ('synchronized_at', models.DateTimeField(null=True, editable=False, blank=True, verbose_name='synchronized')),
                ('username', models.CharField(null=True, blank=True, max_length=32, unique=True)),
                ('custom_id', models.CharField(null=True, blank=True, max_length=48, unique=True)),
                ('enabled', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name_plural': 'Consumer References',
                'verbose_name': 'Consumer Reference',
            },
        ),
        migrations.CreateModel(
            name='KeyAuthReference',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('kong_id', models.UUIDField(null=True, editable=False, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('synchronized', models.BooleanField(default=False)),
                ('synchronized_at', models.DateTimeField(null=True, editable=False, blank=True, verbose_name='synchronized')),
                ('key', models.TextField()),
                ('consumer', models.ForeignKey(to='kong_admin.ConsumerReference')),
            ],
            options={
                'verbose_name_plural': 'Key Auth References',
                'verbose_name': 'Key Auth Reference',
            },
        ),
        migrations.CreateModel(
            name='OAuth2Reference',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('kong_id', models.UUIDField(null=True, editable=False, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('synchronized', models.BooleanField(default=False)),
                ('synchronized_at', models.DateTimeField(null=True, editable=False, blank=True, verbose_name='synchronized')),
                ('name', models.CharField(max_length=32, unique=True)),
                ('redirect_uri', models.URLField()),
                ('client_id', models.CharField(null=True, blank=True, max_length=64, unique=True)),
                ('client_secret', models.TextField(null=True, blank=True)),
                ('consumer', models.ForeignKey(to='kong_admin.ConsumerReference')),
            ],
            options={
                'verbose_name_plural': 'OAuth2 References',
                'verbose_name': 'OAuth2 Reference',
            },
        ),
        migrations.CreateModel(
            name='PluginConfigurationField',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('property', models.CharField(max_length=32)),
                ('value', models.CharField(max_length=32)),
            ],
            options={
                'verbose_name_plural': 'Plugin Configuration Fields',
                'verbose_name': 'Plugin Configuration Field',
            },
        ),
        migrations.CreateModel(
            name='PluginConfigurationReference',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('kong_id', models.UUIDField(null=True, editable=False, blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated')),
                ('synchronized', models.BooleanField(default=False)),
                ('synchronized_at', models.DateTimeField(null=True, editable=False, blank=True, verbose_name='synchronized')),
                ('name', models.CharField(choices=[('basicauth', 'basicauth'), ('cors', 'cors'), ('filelog', 'filelog'), ('httplog', 'httplog'), ('keyauth', 'keyauth'), ('oauth2', 'oauth2'), ('ratelimiting', 'ratelimiting'), ('request_transformer', 'request_transformer'), ('requestsizelimiting', 'requestsizelimiting'), ('response_transformer', 'response_transformer'), ('ssl', 'ssl'), ('tcplog', 'tcplog'), ('udplog', 'udplog')], max_length=32, verbose_name='Plugin Name')),
                ('enabled', models.BooleanField(default=True)),
                ('api', models.ForeignKey(related_name='plugins', to='kong_admin.APIReference')),
                ('consumer', models.ForeignKey(related_name='plugins', blank=True, null=True, to='kong_admin.ConsumerReference')),
            ],
            options={
                'verbose_name_plural': 'Plugin Configuration References',
                'verbose_name': 'Plugin Configuration Reference',
            },
        ),
        migrations.AddField(
            model_name='pluginconfigurationfield',
            name='configuration',
            field=models.ForeignKey(related_name='fields', to='kong_admin.PluginConfigurationReference'),
        ),
        migrations.AddField(
            model_name='basicauthreference',
            name='consumer',
            field=models.ForeignKey(to='kong_admin.ConsumerReference'),
        ),
        migrations.AlterUniqueTogether(
            name='pluginconfigurationreference',
            unique_together=set([('name', 'api')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
import logging

from django.db import transaction
from django.utils import timezone

from six import with_metaclass
from abc import ABCMeta, abstractmethod

logger = logging.getLogger(__name__)


class KongProxySyncEngine(with_metaclass(ABCMeta, object)):
    @abstractmethod
    def get_proxy_class(self):
        """
        :return: Returns the actual class of the KongProxyModel were working with
        """

    @abstractmethod
    def on_retrieve_all(self, client):
        """
        Called to retrieve all objects from kong

        :param client:
        :type client: kong.contract.KongAdminContract
        :return: collections.Iterable
        """

    @abstractmethod
    def is_published(self, client, kong_id, parent_kong_id=None):
        """
        Caleld to check whether an object is known at kong

        :param client:
        :param kong_id:
        :param parent_kong_id:
        :return:
        """

    @abstractmethod
    def on_publish(self, client, obj):
        """
        Called to publish a KongProxyModel to Kong

        :param client:
        :type client: kong.contract.KongAdminContract
        :param obj:
        :type obj: kong_admin.models.KongProxyModel
        :rtype: uuid.UUID
        :return: The uuid of the newly published object
        """

    @abstractmethod
    def on_withdraw_by_id(self, client, kong_id, parent_kong_id=None):
        """
        Called to withdraw an object from Kong by its 'Kong ID'

        :param client:
        :type client: kong.contract.KongAdminContract
        :param kong_id:
        :type kong_id: uuid.UUID
        :param parent_kong_id: Optional reference to a parent object
        :type parent_kong_id: uuid.UUID
        """

    def on_withdraw(self, client, obj):
        """
        Called to withdraw a KongProxyModel from Kong

        :param client:
        :type client: kong.contract.KongAdminContract
        :param obj:
        :type obj: kong_admin.models.KongProxyModel
        """
        parent_object = self.get_parent_object(obj)

        if obj.kong_id is None:
            return obj

        return self.on_withdraw_by_id(
            client, str(obj.kong_id), str(parent_object.kong_id) if parent_object is not None else None)

    def before_publish(self, client, obj):
        parent_object = self.get_parent_object(obj)

        if obj.kong_id is None:
            return

        if not self.is_published(client, obj.kong_id, parent_object.kong_id if parent_object is not None else None):
            obj.kong_id = None
            self.get_proxy_class().objects.filter(id=obj.id).update(kong_id=obj.kong_id)

    def after_publish(self, client, obj):
        obj.synchronized_at = timezone.now()
        obj.synchronized = True

        # Doing this instead of saving will prevent the save signal from being send out!!!
        self.get_proxy_class().objects.filter(id=obj.id).update(
            synchronized=obj.synchronized, synchronized_at=obj.synchronized_at)

    def before_withdraw(self, client, obj):
        pass

    def after_withdraw(self, client, obj):
        obj.synchronized_at = None
        obj.synchronized = False

        # Doing this instead of saving will prevent the save signal from being send out!!!
        self.get_proxy_class().objects.filter(id=obj.id).update(
            synchronized=obj.synchronized, synchronized_at=obj.synchronized_at)

    def get_parent_object(self, obj):
        """
        Returns a parent object for a given object

        :param obj:
        :return:
        """

    def get_parent_key(self):
        """
        Returns the key that references the parent object
        :return:
        """

    def publish(self, client, obj):
        """
        Publish a KongProxyModel to Kong

        :param client:
        :type client: kong.contract.KongAdminContract
        :param obj:
        :type obj: kong_admin.models.KongProxyModel
        :rtype: kong_admin.models.KongProxyModel
        :return: The KongProxyModel that has been published to Kong
        """
        with transaction.atomic():
            self.before_publish(client, obj)

        kong_id = self.on_publish(client, obj)

        # Always update the kong_id
        obj.kong_id = kong_id
        self.get_proxy_class().objects.filter(id=obj.id).update(kong_id=obj.kong_id)

        with transaction.atomic():
            self.after_publish(client, obj)

        return obj

    def withdraw(self, client, obj):
        """
        Withdraw a KongProxy model from Kong

        :param client:
        :type client: kong.contract.KongAdminContract
        :param obj:
        :type obj: kong_admin.models.KongProxyModel
        :rtype: kong_admin.models.KongProxyModel
        :return: The KongProxyModel that has been withdrawn from Kong
        """
        with transaction.atomic():
            self.before_withdraw(client, obj)

        self.on_withdraw(client, obj)

        # Always update the kong_id
        obj.kong_id = None
        self.get_proxy_class().objects.filter(id=obj.id).update(kong_id=obj.kong_id)

        with transaction.atomic():
            self.after_withdraw(client, obj)

        return obj

    def withdraw_by_id(self, client, kong_id, parent_kong_id=None):
        """
        Withdraw an object from Kong by its 'Kong ID'

        :param client:
        :type client: kong.contract.KongAdminContract
        :param kong_id: The id of the object, as it is known by Kong
        :type kong_id: uuid.UUID
        :rtype: kong_admin.models.KongProxyModel
        :return: The kong_id of the object that has been withdrawn from Kong
        """
        try:
            obj = self.get_proxy_class().objects.get(kong_id=kong_id)
        except self.get_proxy_class().DoesNotExist:
            obj = None

        if obj is not None:
            return self.withdraw(client, obj)

        # We don't have a reference to that API anymore. Just try to remove it remotely
        self.on_withdraw_by_id(client, kong_id, parent_kong_id)
        return obj

    def synchronize(self, client, queryset=None, delete=False):
        """
        :param client: The client to use
        :type client: kong.contract.KongAdminContract
        :param queryset: A queryset containing KongProxyModel objects
        :type queryset: django.db.models.QuerySet.
        :param delete: Whether or not to delete the object in the Kong service, if there is no model.
        :type delete: bool
        :return:
        """

        # Make sure we have a queryset
        queryset = queryset or self.get_proxy_class().objects.all()

        # Delete remote api's that do not exist in this database
        if delete:
            for kong_struct in self.on_retrieve_all(client):
                kong_id = kong_struct.get('id', None)
                assert kong_id is not None

                parent_kong_id = kong_struct.get(self.get_parent_key(), None)

                if not queryset.filter(kong_id=kong_id).exists():
                    logger.debug('synchronize: delete %s by id: %s' % (self.get_proxy_class(), kong_id))
                    self.withdraw_by_id(client, kong_id, parent_kong_id=parent_kong_id)

        # Add remote apis that only exist in this database
        for obj in queryset:
            self.publish(client, obj)

        return queryset



# -*- coding: utf-8 -*-
import logging
from six import python_2_unicode_compatible

from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from jsonfield2 import JSONField, JSONAwareManager

from kong.exceptions import ConflictError
from .enums import Plugins

logger = logging.getLogger(__name__)


class KongProxyModel(models.Model):
    kong_id = models.UUIDField(null=True, blank=True, editable=False)

    created_at = models.DateTimeField(_('created'), auto_now_add=True)
    updated_at = models.DateTimeField(_('updated'), auto_now=True)
    synchronized = models.BooleanField(default=False)
    synchronized_at = models.DateTimeField(_('synchronized'), null=True, blank=True, editable=False)

    class Meta:
        abstract = True


@python_2_unicode_compatible
class APIReference(KongProxyModel):
    target_url = models.URLField()
    name = models.CharField(null=True, blank=True, unique=True, max_length=32, default=None)
    public_dns = models.CharField(null=True, blank=True, unique=True, max_length=32, default=None)
    path = models.CharField(null=True, blank=True, max_length=32, default=None)
    strip_path = models.BooleanField(default=False)
    enabled = models.BooleanField(default=True)

    class Meta:
        verbose_name = _('API Reference')
        verbose_name_plural = _('API References')

    def __str__(self):
        return self.target_url if not self.name else '%s (%s)' % (self.name, self.target_url)

    def clean(self):
        self.name = self.name or None  # Don't store empty strings
        self.public_dns = self.public_dns or None  # Don't store empty strings
        self.path = self.path or None  # Don't store empty strings

        if not self.public_dns and not self.path:
            raise ValidationError('At least one of the parameters "public_dns" and "path" should be set')

        if self.synchronized_at and not self.kong_id:
            raise ValidationError('There should be an kong_id parameter')

        if self.kong_id and not self.synchronized_at:
            raise ValidationError('There should be a synchronized_at parameter')


@python_2_unicode_compatible
class PluginConfigurationReference(KongProxyModel):
    api = models.ForeignKey(APIReference, related_name='plugins')
    consumer = models.ForeignKey('ConsumerReference', null=True, blank=True, related_name='plugins')
    name = models.CharField(_('Plugin Name'), choices=Plugins.choices(), max_length=32)
    enabled = models.BooleanField(default=True)
    value = JSONField(default={})

    objects = JSONAwareManager(json_fields=['value'])

    class Meta:
        verbose_name = _('Plugin Configuration Reference')
        verbose_name_plural = _('Plugin Configuration References')
        unique_together = [('name', 'api')]

    def __str__(self):
        return self.name


# @python_2_unicode_compatible
# class PluginConfigurationField(models.Model):
#     configuration = models.ForeignKey(PluginConfigurationReference, related_name='fields')
#     property = models.CharField(max_length=32)
#     value = models.CharField(max_length=32)
#
#     class Meta:
#         verbose_name = _('Plugin Configuration Field')
#         verbose_name_plural = _('Plugin Configuration Fields')
#
#     def __str__(self):
#         return '%s = %s' % (self.property, self.value)


@python_2_unicode_compatible
class ConsumerReference(KongProxyModel):
    username = models.CharField(null=True, blank=True, unique=True, max_length=32)
    custom_id = models.CharField(null=True, blank=True, unique=True, max_length=48)
    enabled = models.BooleanField(default=True)

    class Meta:
        verbose_name = _('Consumer Reference')
        verbose_name_plural = _('Consumer References')

    def __str__(self):
        return self.username or self.custom_id

    def clean(self):
        self.username = self.username or None  # Don't store empty strings
        self.custom_id = self.custom_id or None  # Don't store empty strings

        if not self.username and not self.custom_id:
            raise ValidationError('At least one of the parameters "username" and "custom_id" should be set')


class ConsumerAuthentication(KongProxyModel):
    consumer = models.ForeignKey(ConsumerReference)

    class Meta:
        abstract = True


@python_2_unicode_compatible
class BasicAuthReference(ConsumerAuthentication):
    username = models.CharField(unique=True, max_length=32)
    password = models.CharField(max_length=40)

    class Meta:
        verbose_name = _('Basic Auth Reference')
        verbose_name_plural = _('Basic Auth References')

    def __str__(self):
        return 'BasicAuthReference(consumer: %s, username: %s)' % (self.consumer, self.username)


@python_2_unicode_compatible
class KeyAuthReference(ConsumerAuthentication):
    key = models.TextField()

    class Meta:
        verbose_name = _('Key Auth Reference')
        verbose_name_plural = _('Key Auth References')

    def __str__(self):
        key = self.key
        if len(key) > 16:
            key = '%s...' % key[:16]
        return 'KeyAuthReference(consumer: %s, key: %s)' % (self.consumer, key)


@python_2_unicode_compatible
class OAuth2Reference(ConsumerAuthentication):
    name = models.CharField(unique=True, max_length=32)
    redirect_uri = models.URLField()
    client_id = models.CharField(null=True, blank=True, unique=True, max_length=64)
    client_secret = models.TextField(null=True, blank=True)

    class Meta:
        verbose_name = _('OAuth2 Reference')
        verbose_name_plural = _('OAuth2 References')

    def __str__(self):
        return 'OAuth2Reference(name: %s)' % self.name

    def clean(self):
        self.client_id = self.client_id or None  # Don't store empty strings
        self.client_secret = self.client_secret or None  # Don't store empty strings



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from pip.req import parse_requirements

import kong_admin

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = kong_admin.__version__

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on github:")
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

readme = open(os.path.join(BASE_DIR, 'README.rst'),).read()
history = open(os.path.join(BASE_DIR, 'CHANGELOG.rst')).read().replace('.. :changelog:', '')

requirements = [str(ir.req) for ir in parse_requirements(os.path.join(BASE_DIR, 'requirements.txt'), session=False)]

setup(
    name='django-kong-admin',
    version=version,
    description="""A reusable Django App to manage a Kong service (http://getkong.org)""",
    long_description=readme + '\n\n' + history,
    author='Dirk Moors',
    author_email='dirk.moors@vikingco.com',
    url='https://github.com/vikingco/django-kong-admin',
    packages=[
        'kong_admin',
    ],
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='django-kong-admin',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)
